#!/usr/bin/env node
"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));

// src/cli.ts
var import_cac = __toESM(require("cac"));
var import_bundle_require = require("bundle-require");

// src/util.ts
var import_sync = __toESM(require("escalade/sync"));
var import_fs = require("fs");
var import_path = __toESM(require("path"));
function getOutputFile(file, format) {
  const id = Math.random().toString(36).substring(2, 15);
  const cacheDir = import_path.default.join(nearestNodeModules(file), ".cache/codegentool");
  const outputFile = import_path.default.join(
    cacheDir,
    file.replace(
      /\.([mc]?[tj]s|[tj]sx)$/,
      `.bundled_${id}.${format === "esm" ? "mjs" : "cjs"}`
    )
  );
  (0, import_fs.mkdirSync)(import_path.default.dirname(outputFile), { recursive: true });
  return outputFile;
}
function nearestNodeModules(file) {
  const dir = (0, import_sync.default)(import_path.default.dirname(file), (dir2, names) => {
    if (names.includes("node_modules")) {
      return dir2;
    }
  });
  if (dir) {
    return import_path.default.join(dir, "node_modules");
  }
  return "node_modules";
}

// src/cli.ts
var import_chokidar = require("chokidar");

// src/config.ts
var import_sync2 = __toESM(require("escalade/sync"));
var import_fs2 = __toESM(require("fs"));
function readConfig(cwd) {
  let config;
  let configPath;
  const watchPaths = [];
  (0, import_sync2.default)(cwd, (dir, names) => {
    watchPaths.push(dir + "/package.json");
    if (names.includes("package.json")) {
      let pkg;
      try {
        pkg = JSON.parse(import_fs2.default.readFileSync(dir + "/package.json", "utf8"));
      } catch {
      }
      if (pkg?.codegentool && typeof pkg.codegentool === "object") {
        config = pkg.codegentool;
        configPath = dir + "/package.json";
        return dir;
      }
    }
    if (names.includes(".git")) {
      return dir;
    }
  });
  config = {
    generators: ["generators/**/*.{ts,mts,js,mjs}", "!**/node_modules/**"],
    format: "cjs",
    ...config
  };
  return {
    config,
    configPath,
    watchPaths
  };
}

// src/cli.ts
var import_dequal = require("dequal");
var import_fs3 = __toESM(require("fs"));
var import_path2 = __toESM(require("path"));
var import_fast_glob = __toESM(require("fast-glob"));
var import_dedent = __toESM(require("dedent"));
var import_kleur = __toESM(require("kleur"));
var program = (0, import_cac.default)("codegentool");
program.command("").option("--watch, -w", "Enable file watching and automatic reruns").action((options) => start(process.cwd(), options));
program.parse();
async function start(cwd, options) {
  const { config, configPath, watchPaths } = readConfig(cwd);
  if (configPath) {
    process.chdir(import_path2.default.dirname(configPath));
  }
  const watcher = (0, import_chokidar.watch)([...config.generators, ...watchPaths]);
  const generators = /* @__PURE__ */ new Map();
  let initialized = false;
  watcher.on("all", (event, name) => {
    if (name.endsWith("package.json")) {
      if (!initialized)
        return;
      const configResult = readConfig(process.cwd());
      if (!(0, import_dequal.dequal)(config, configResult.config) || !(0, import_dequal.dequal)(watchPaths, configResult.watchPaths)) {
        watcher.close();
        generators.forEach(async (close) => (await close)());
        start(cwd, options);
      }
    } else if (event == "add" || event == "change") {
      const promise = generators.get(name) ?? Promise.resolve();
      promise.then((close) => close?.()).then(() => {
        generators.set(name, generate(name, config));
      });
    }
  });
  await new Promise((resolve) => {
    watcher.once("ready", () => {
      initialized = true;
      resolve();
    });
  });
  if (options.watch) {
    console.log(`\u2318 Watching for changes...`);
  } else {
    watcher.close();
    generators.forEach(async (close) => (await close)());
  }
}
async function generate(generatorPath, config) {
  const watcher = (0, import_chokidar.watch)([generatorPath], {
    ignoreInitial: true
  });
  let close = Promise.resolve(() => watcher.close());
  watcher.on(
    "all",
    debounce(() => {
      watcher.close();
      close = generate(generatorPath, config);
    }, 500)
  );
  try {
    console.log(
      `\u25B6 Running %s`,
      import_kleur.default.cyan(import_path2.default.relative(process.cwd(), generatorPath))
    );
    const {
      mod: { default: generator },
      dependencies
    } = await (0, import_bundle_require.bundleRequire)({
      filepath: generatorPath,
      getOutputFile,
      tsconfig: config.tsconfig,
      format: config.format
    });
    if (typeof generator !== "function") {
      throw new Error(`Generator must default export a function`);
    }
    watcher.add(dependencies);
    let filesGenerated = 0;
    const watchPaths = /* @__PURE__ */ new Set();
    const api = {
      scan: (source, options) => {
        const cwd = import_path2.default.resolve(options?.cwd || ".");
        if (Array.isArray(source)) {
          source.forEach((s) => watchPaths.add(import_path2.default.resolve(cwd, s)));
        } else {
          watchPaths.add(import_path2.default.resolve(cwd, source));
        }
        return import_fast_glob.default.sync(source, options);
      },
      read: (file, encoding) => {
        watchPaths.add(file);
        return import_fs3.default.readFileSync(file, encoding);
      },
      write: (file, data) => {
        if (typeof data === "string") {
          let current;
          try {
            current = import_fs3.default.readFileSync(file, "utf8");
          } catch {
          }
          if (data !== current) {
            import_fs3.default.mkdirSync(import_path2.default.dirname(file), { recursive: true });
            import_fs3.default.writeFileSync(file, data);
            filesGenerated++;
            console.log(
              `\u2714\uFE0F Generated %s`,
              import_kleur.default.green(import_path2.default.relative(process.cwd(), file))
            );
          }
        } else {
          let current;
          try {
            current = import_fs3.default.readFileSync(file);
          } catch {
          }
          if (!current?.equals(data)) {
            import_fs3.default.mkdirSync(import_path2.default.dirname(file), { recursive: true });
            import_fs3.default.writeFileSync(file, data);
            filesGenerated++;
            console.log(
              `\u2714\uFE0F Generated %s`,
              import_kleur.default.green(import_path2.default.relative(process.cwd(), file))
            );
          }
        }
      },
      dedent: import_dedent.default
    };
    await generator(api);
    if (!filesGenerated) {
      console.log(
        `\u2714\uFE0F Nothing updated by %s`,
        import_kleur.default.cyan(import_path2.default.relative(process.cwd(), generatorPath))
      );
    }
    watcher.add([...watchPaths]);
  } catch (error) {
    console.error(`Error in ${import_path2.default.resolve(generatorPath)}`);
    console.error(error);
  }
  return async () => (await close)();
}
function debounce(fn, wait) {
  let timeout;
  return function(...args) {
    clearTimeout(timeout);
    timeout = setTimeout(() => fn.apply(this, args), wait);
  };
}
